@@ -56,6 +56,8 @@ static void	   gtk_game_info_dialog_class_init
 		     (GtkGameInfoDialogClass *class);
 static void	   gtk_game_info_dialog_init (GtkGameInfoDialog *dialog);
 
+static void	   update_title (GtkGameInfoDialog *dialog);
+
 static GtkEntry *  create_and_pack_game_info_entry (const gchar *label_text,
 						    SgfType sgf_property_type,
 						    GtkWidget **hbox,
@@ -221,9 +223,6 @@ gtk_game_info_dialog_init (GtkGameInfoDialog *dialog)
   GtkSizeGroup *size_group;
   int k;
 
-  /* FIXME: Game name in title (if present). */
-  gtk_window_set_title (GTK_WINDOW (dialog), _("Game Information"));
-
   dialog->pages = GTK_NOTEBOOK (notebook);
 
   for (k = 0; k < NUM_COLORS; k++) {
@@ -507,10 +506,10 @@ gtk_game_info_dialog_init (GtkGameInfoDialog *dialog)
   gtk_dialog_set_has_separator (&dialog->dialog, FALSE);
 
   gtk_dialog_add_buttons (&dialog->dialog,
-			  GTK_STOCK_HELP, GTK_RESPONSE_HELP,
 			  GTK_STOCK_UNDO, GTK_GAME_INFO_DIALOG_RESPONSE_UNDO,
 			  GTK_STOCK_REDO, GTK_GAME_INFO_DIALOG_RESPONSE_REDO,
 			  GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE, NULL);
+  gtk_utils_add_help_button (&dialog->dialog);
 
   dialog->sgf_tree	    = NULL;
   dialog->sgf_node	    = NULL;
@@ -519,6 +518,27 @@ gtk_game_info_dialog_init (GtkGameInfoDialog *dialog)
   dialog->simple_undo_field = NULL;
   dialog->simple_redo_field = NULL;
   dialog->simple_redo_value = NULL;
+
+  update_title (dialog);
+}
+
+
+static void
+update_title (GtkGameInfoDialog *dialog)
+{
+  if (dialog->sgf_node
+      && sgf_node_get_text_property_value (dialog->sgf_node, SGF_GAME_NAME)) {
+    gchar* title
+      = g_strdup_printf ("%s - %s",
+			 sgf_node_get_text_property_value (dialog->sgf_node,
+							   SGF_GAME_NAME),
+			 _("Game Information"));
+
+    gtk_window_set_title (GTK_WINDOW (dialog), title);
+    g_free (title);
+  }
+  else
+    gtk_window_set_title (GTK_WINDOW (dialog), _("Game Information"));
 }
 
 
@@ -777,6 +797,7 @@ get_field (GtkGameInfoDialog *dialog, SgfType sgf_property_type)
 
   default:
     g_assert_not_reached ();
+    return NULL;
   }
 }
 
@@ -816,6 +837,7 @@ get_page_index (SgfType sgf_property_type)
 
   default:
     g_assert_not_reached ();
+    return -1;
   }
 }
 
@@ -893,6 +915,7 @@ get_property_type (GtkGameInfoDialog *dialog, void *field)
     return SGF_USER;
 
   g_assert_not_reached ();
+  return SGF_UNKNOWN;
 }
 
 
@@ -912,8 +935,10 @@ get_field_text (void *field)
 				&start_iterator, &end_iterator);
     return gtk_text_iter_get_text (&start_iterator, &end_iterator);
   }
-  else
+  else {
     g_assert_not_reached ();
+    return NULL;
+  }
 }
 
 
@@ -952,6 +977,8 @@ set_field_text (GtkGameInfoDialog *dialog, SgfType sgf_property_type)
       goto text_set;
     }
   }
+  else if (sgf_property_type == SGF_GAME_NAME)
+    update_title (dialog);
 
   do_set_field_text (field,
 		     sgf_node_get_text_property_value (dialog->sgf_node,
@@ -1099,8 +1126,10 @@ game_info_spin_button_focus_out_event
 	sgf_property_type = SGF_KOMI;
       else if (dialog->main_time_spin_button == freezable_spin_button)
 	sgf_property_type = SGF_TIME_LIMIT;
-      else
+      else {
 	g_assert_not_reached ();
+	return FALSE;
+      }
 
       update_property (dialog, sgf_property_type, NULL);
     }
@@ -1138,14 +1167,21 @@ undo_or_redo_availability_changed (SgfUndoHistory *undo_history,
 				   void *user_data)
 {
   GtkGameInfoDialog *dialog = GTK_GAME_INFO_DIALOG (user_data);
-
-  gboolean can_undo = (dialog->simple_undo_field
-		       || sgf_utils_can_undo (dialog->sgf_tree));
-  gboolean can_redo = (dialog->simple_redo_field
-		       || sgf_utils_can_redo (dialog->sgf_tree));
+  SgfUndoHistory *saved_undo_history = dialog->sgf_tree->undo_history;
+  gboolean can_undo;
+  gboolean can_redo;
 
   UNUSED (undo_history);
 
+  dialog->sgf_tree->undo_history = dialog->sgf_undo_history;
+
+  can_undo = (dialog->simple_undo_field
+	      || sgf_utils_can_undo (dialog->sgf_tree));
+  can_redo = (dialog->simple_redo_field
+	      || sgf_utils_can_redo (dialog->sgf_tree));
+
+  dialog->sgf_tree->undo_history = saved_undo_history;
+
   gtk_dialog_set_response_sensitive (&dialog->dialog,
 				     GTK_GAME_INFO_DIALOG_RESPONSE_UNDO,
 				     can_undo);
@@ -1184,6 +1220,9 @@ update_property (GtkGameInfoDialog *dialog, SgfType sgf_property_type,
 		     sgf_property_type);
 
       dialog->simple_undo_field = NULL;
+
+      if (sgf_property_type == SGF_GAME_NAME)
+	update_title (dialog);
     }
 
     sgf_utils_end_action (sgf_tree);
