@@ -32,6 +32,8 @@
 #include <stdarg.h>
 #include <string.h>
 
+#include <gdk/gdkkeysyms.h>
+
 
 typedef struct _GtkUtilsBrowseButtonData	GtkUtilsBrowseButtonData;
 
@@ -154,6 +156,22 @@ gtk_utils_standardize_dialog (GtkDialog *dialog, GtkWidget *contents)
 }
 
 
+GtkWidget*
+gtk_utils_add_help_button (GtkDialog* dialog)
+{
+  GtkAccelGroup* accel_group = gtk_accel_group_new ();
+  GtkWidget* help_button = gtk_dialog_add_button (dialog, GTK_STOCK_HELP,
+						  GTK_RESPONSE_HELP);
+
+  gtk_widget_add_accelerator (help_button, "clicked",
+			      accel_group, GDK_F1, 0, 0);
+
+  gtk_window_add_accel_group (GTK_WINDOW (dialog), accel_group);
+
+  return help_button;
+}
+
+
 void
 gtk_utils_show_and_forget_dialog (GtkDialog *dialog)
 {
@@ -255,6 +273,7 @@ g_signal_accumulator_true_handled (GSignalInvocationHint *hint,
 {
   gboolean signal_handled;
 
+  UNUSED (hint);
   UNUSED (user_data);
   
   signal_handled = g_value_get_boolean (handler_return);
