@@ -128,6 +128,7 @@ quarry_find_dialog_init (QuarryFindDialog *dialog)
 			  QUARRY_FIND_DIALOG_FIND_PREVIOUS,
 			  QUARRY_STOCK_FIND_NEXT,
 			  QUARRY_FIND_DIALOG_FIND_NEXT, NULL);
+  gtk_utils_add_help_button (&dialog->dialog);
   gtk_dialog_set_default_response (&dialog->dialog,
 				   QUARRY_FIND_DIALOG_FIND_NEXT);
 
@@ -147,8 +148,6 @@ quarry_find_dialog_init (QuarryFindDialog *dialog)
 
 #endif /* not GTK_2_4_OR_LATER */
 
-  /* FIXME: Implement history, maybe only for GTK+ 2.4. */
-
   g_signal_connect_swapped (entry, "changed",
 			    G_CALLBACK (parameters_changed), dialog);
 
@@ -387,6 +386,11 @@ quarry_find_dialog_set_search_history (QuarryFindDialog *dialog,
   for (item = strings->first; item; item = item->next)
     gtk_combo_box_append_text (combo_box, item->text);
 
+#else  /* not GTK_2_4_OR_LATER */
+
+  UNUSED (dialog);
+  UNUSED (strings);
+
 #endif /* not GTK_2_4_OR_LATER */
 }
 
@@ -575,6 +579,7 @@ quarry_find_text (const gchar *text_to_find,
 
     default:
       g_assert_not_reached ();
+      return FALSE;
     }
 
     while (1) {
