@@ -1,7 +1,7 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
  * This file is part of Quarry.                                    *
  *                                                                 *
- * Copyright (C) 2003, 2004, 2005 Paul Pogonyshev                  *
+ * Copyright (C) 2003, 2004, 2005, 2006 Paul Pogonyshev.           *
  *                                                                 *
  * This program is free software; you can redistribute it and/or   *
  * modify it under the terms of the GNU General Public License as  *
@@ -144,6 +144,7 @@ static void	    parse_unknown_property_values
 
 static int	    do_parse_number (SgfParsingData *data, int *number);
 static int	    do_parse_real (SgfParsingData *data, double *real);
+inline static int   do_parse_color (SgfParsingData *data);
 
 static char *	    do_parse_simple_text (SgfParsingData *data,
 					  char extra_stop_character);
@@ -190,7 +191,8 @@ static int	    is_composed_value (SgfParsingData *data,
 				       int expecting_simple_text);
 static SgfError	    end_parsing_value (SgfParsingData *data);
 
-static int	    format_error_valist (SgfParsingData *data, char *buffer,
+static void	    format_error_valist (SgfParsingData *data,
+					 StringBuffer *buffer,
 					 SgfError error, va_list arguments);
 
 
@@ -356,7 +358,9 @@ parse_buffer (SgfParsingData *data,
   else
     data->bytes_parsed = &dummy_bytes_parsed;
 
-  data->cancelled = 0;
+  data->file_error = 0;
+  data->cancelled  = 0;
+
   if (cancellation_flag)
     data->cancellation_flag = cancellation_flag;
   else {
@@ -417,7 +421,10 @@ parse_buffer (SgfParsingData *data,
     sgf_collection_delete (*collection);
     *collection = NULL;
 
-    return data->cancelled ? SGF_PARSING_CANCELLED : SGF_INVALID_FILE;
+    if (data->file_error)
+      return SGF_ERROR_READING_FILE;
+    else
+      return data->cancelled ? SGF_PARSING_CANCELLED : SGF_INVALID_FILE;
   }
 
   if (string_list_is_empty (*error_list)) {
@@ -859,11 +866,17 @@ refresh_buffer (SgfParsingData *data)
     data->buffer_refresh_point = data->buffer_end;
   }
 
-  /* FIXME: add less severe handling of errors. */
   if (fread ((data->buffer + 1) + unused_bytes, bytes_to_read, 1,
 	     data->file)
-      != 1)
-    assert (0);
+      != 1) {
+    data->file_error	       = 1;
+    data->cancelled	       = 1;
+    data->buffer_end	       = data->buffer_pointer;
+    data->buffer_refresh_point = data->buffer_end;
+    data->token		       = SGF_END;
+
+    return;
+  }
 
   data->buffer_pointer	       = data->buffer + 1;
   data->file_bytes_remaining  -= bytes_to_read;
@@ -891,11 +904,17 @@ expand_buffer (SgfParsingData *data)
 				data->buffer_size + buffer_increase);
   data->buffer_pointer = data->buffer + data->buffer_size;
 
-  /* FIXME: add less severe handling of errors. */
   if (fread ((char *) data->buffer + data->buffer_size,
 	     buffer_increase, 1, data->file)
-      != 1)
-    assert (0);
+      != 1) {
+    data->file_error	       = 1;
+    data->cancelled	       = 1;
+    data->buffer_end	       = data->buffer_pointer;
+    data->buffer_refresh_point = data->buffer_end;
+    data->token		       = SGF_END;
+
+    return;
+  }
 
   data->buffer_size		  += buffer_increase;
   data->buffer_end		   = data->buffer + data->buffer_size;
@@ -985,14 +1004,13 @@ complete_node_and_update_board (SgfParsingData *data, int is_leaf_node)
     if (data->node->move_color != EMPTY
 	|| sgf_node_find_property (data->node, SGF_MOVE_NUMBER, NULL)) {
       if (has_setup_add_properties
-	  || (sgf_node_get_color_property_value (data->node, SGF_TO_PLAY)
-	      == data->node->move_color)) {
+	  || data->node->to_play_color == data->node->move_color) {
 	sgf_node_split (data->node, data->tree);
 	insert_error (data, SGF_ERROR_MIXED_SETUP_ADD_AND_MOVE_PROPERTIES,
 		      &data->node_error_position);
       }
       else {
-	sgf_node_delete_property (data->node, data->tree, SGF_TO_PLAY);
+	data->node->to_play_color = EMPTY;
 	insert_error (data, SGF_ERROR_MIXED_PL_AND_MOVE_PROPERTIES,
 		      &data->node_error_position);
       }
@@ -1424,6 +1442,26 @@ sgf_parse_double (SgfParsingData *data)
 }
 
 
+static int
+do_parse_color (SgfParsingData *data)
+{
+  if (data->token == 'B')
+    return BLACK;
+
+  if (data->token == 'W')
+    return WHITE;
+
+  if (data->token == 'b' || data->token == 'w') {
+    add_error (data, SGF_WARNING_LOWER_CASE_COLOR,
+	       data->token, data->token - ('b' - 'B'));
+
+    return data->token == 'b' ? BLACK : WHITE;
+  }
+
+  return EMPTY;
+}
+
+
 /* Parse color value.  [W] stands for white and [B] - for black.  No
  * other values are allowed except that [w] and [b] are upcased and
  * warned about.
@@ -1432,6 +1470,7 @@ SgfError
 sgf_parse_color (SgfParsingData *data)
 {
   SgfProperty **link;
+  int color;
 
   if (sgf_node_find_property (data->node, data->property_type, &link))
     return SGF_FATAL_DUPLICATE_PROPERTY;
@@ -1440,21 +1479,11 @@ sgf_parse_color (SgfParsingData *data)
   if (data->token == ']')
     return SGF_FATAL_EMPTY_VALUE;
 
-  if (data->token == 'b' || data->token == 'w') {
-    char original_color = data->token;
+  color = do_parse_color (data);
 
-    data->token -= ('b' - 'B');
-    add_error (data, SGF_WARNING_LOWER_CASE_COLOR,
-	       original_color, data->token);
-  }
-
-  if (data->token == 'B' || data->token == 'W') {
+  if (color != EMPTY) {
     *link = sgf_property_new (data->tree, data->property_type, *link);
-    (*link)->value.color = (data->token == 'B' ? BLACK : WHITE);
-
-    /* `PL' is a setup property. */
-    if (data->property_type == SGF_TO_PLAY)
-      data->has_any_setup_property = 1;
+    (*link)->value.color = color;
 
     next_token_in_value (data);
     return end_parsing_value (data);
@@ -2841,6 +2870,30 @@ sgf_parse_time_limit (SgfParsingData *data)
 }
 
 
+SgfError
+sgf_parse_to_play (SgfParsingData *data)
+{
+  if (data->node->to_play_color != EMPTY)
+    return SGF_FATAL_DUPLICATE_PROPERTY;
+
+  begin_parsing_value (data);
+  if (data->token == ']')
+    return SGF_FATAL_EMPTY_VALUE;
+
+  data->node->to_play_color = do_parse_color (data);
+
+  if (data->node->to_play_color != EMPTY) {
+    /* `PL' is a setup property. */
+    data->has_any_setup_property = 1;
+
+    next_token_in_value (data);
+    return end_parsing_value (data);
+  }
+
+  return SGF_FATAL_INVALID_VALUE;
+}
+
+
 /* FIXME: write this function. */
 SgfError
 sgf_parse_letters (SgfParsingData *data)
@@ -2899,18 +2952,20 @@ add_error (SgfParsingData *data, SgfError error, ...)
 
   if (error < SGF_FIRST_PROPERTY_NAME_ERROR
       || SGF_LAST_PROPERTY_NAME_ERROR < error) {
-    char buffer[MAX_ERROR_LENGTH];
-    int length;
+    StringBuffer buffer;
 
     if (error != SGF_WARNING_ERROR_SUPPRESSED
 	&& data->times_error_reported[error] == MAX_TIMES_TO_REPORT_ERROR)
       return;
 
+    string_buffer_init (&buffer, 0x200, 0x200);
+
     va_start (arguments, error);
-    length = format_error_valist (data, buffer, error, arguments);
+    format_error_valist (data, &buffer, error, arguments);
     va_end (arguments);
 
-    string_list_add_from_buffer (data->error_list, buffer, length);
+    string_list_add_ready (data->error_list,
+			   string_buffer_steal_string (&buffer));
     data->error_list->last->line = data->line;
     data->error_list->last->column = data->column + data->first_column;
 
@@ -2944,18 +2999,18 @@ insert_error_valist (SgfParsingData *data, SgfError error,
 		     SgfErrorPosition *error_position, va_list arguments)
 {
   SgfErrorListItem *error_item;
-  char buffer[MAX_ERROR_LENGTH];
-  int length;
+  StringBuffer buffer;
 
   if (error != SGF_WARNING_ERROR_SUPPRESSED
       && data->times_error_reported[error] == MAX_TIMES_TO_REPORT_ERROR)
     return;
 
-  length = format_error_valist (data, buffer, error, arguments);
+  string_buffer_init (&buffer, 0x200, 0x200);
+  format_error_valist (data, &buffer, error, arguments);
 
-  error_item = string_list_insert_from_buffer (data->error_list,
-					       error_position->notch,
-					       buffer, length);
+  error_item = string_list_insert_ready (data->error_list,
+					 error_position->notch,
+					 string_buffer_steal_string (&buffer));
   error_item->line   = error_position->line;
   error_item->column = error_position->column + data->first_column;
 
@@ -3080,23 +3135,21 @@ end_parsing_value (SgfParsingData *data)
  * - `%M': move, stored in `data->node' is formatted according to
  *   current game.
  */
-static int
-format_error_valist (SgfParsingData *data, char *buffer,
+static void
+format_error_valist (SgfParsingData *data, StringBuffer *buffer,
 		     SgfError error, va_list arguments)
 {
-  char *pointer = buffer;
   const char *format_string = sgf_errors[error];
 
   while (*format_string) {
     if (*format_string != '%')
-      *pointer++ = *format_string++;
+      string_buffer_add_character (buffer, *format_string++);
     else {
       format_string++;
       switch (*format_string++) {
       case 'N':
-	strcpy (pointer, property_info[data->property_type].name);
-	pointer += strlen (property_info[data->property_type].name);
-
+	string_buffer_cat_string (buffer,
+				  property_info[data->property_type].name);
 	break;
 
       case 'V':
@@ -3123,20 +3176,14 @@ format_error_valist (SgfParsingData *data, char *buffer,
 	}
 
 	if (data->temp_buffer - data->buffer <= 27) {
-	  memcpy (pointer, data->buffer, data->temp_buffer - data->buffer);
-	  pointer += data->temp_buffer - data->buffer;
+	  string_buffer_cat_as_string (buffer, data->buffer,
+				       data->temp_buffer - data->buffer);
 	}
 	else {
 	  /* Value is long, only output the beginning and the end. */
-	  memcpy (pointer, data->buffer, 12);
-	  pointer += 12;
-
-	  *pointer++ = '.';
-	  *pointer++ = '.';
-	  *pointer++ = '.';
-
-	  memcpy (pointer, data->temp_buffer - 12, 12);
-	  pointer += 12;
+	  string_buffer_cat_as_string (buffer, data->buffer, 12);
+	  string_buffer_add_characters (buffer, '.', 3);
+	  string_buffer_cat_as_string (buffer, data->temp_buffer - 12, 12);
 	}
 
 	break;
@@ -3146,46 +3193,37 @@ format_error_valist (SgfParsingData *data, char *buffer,
 	  int x = va_arg (arguments, int);
 	  int y = va_arg (arguments, int);
 
-	  pointer += game_format_point (data->game,
-					data->board_width, data->board_height,
-					pointer, x, y);
+	  game_format_point (data->game, data->board_width, data->board_height,
+			     buffer, x, y);
 	}
 
 	break;
 
       case 'M':
-	pointer += sgf_utils_format_node_move (data->tree, data->node, pointer,
-					       "B ", "W ", NULL);
+	sgf_utils_format_node_move (data->tree, data->node, buffer,
+				    "B ", "W ", NULL);
 	break;
 
       case 'c':
-	*pointer++ = (char) va_arg (arguments, int);
+	string_buffer_add_character (buffer, (char) va_arg (arguments, int));
 	break;
 
       case 'd':
-	pointer += sprintf (pointer, "%d", va_arg (arguments, int));
+	string_buffer_printf (buffer, "%d", va_arg (arguments, int));
 	break;
 
       case 's':
-	{
-	  const char *string = va_arg (arguments, const char *);
-
-	  strcpy (pointer, string);
-	  pointer += strlen (string);
-	}
-
+	string_buffer_cat_string (buffer, va_arg (arguments, const char *));
 	break;
 
       case 0:
 	break;
 
       default:
-	*pointer++ = *(format_string - 1);
+	string_buffer_add_character (buffer, *(format_string - 1));
       }
     }
   }
-
-  return pointer - buffer;
 }
 
 
