@@ -1,7 +1,7 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
  * This file is part of Quarry.                                    *
  *                                                                 *
- * Copyright (C) 2005 Paul Pogonyshev.                             *
+ * Copyright (C) 2005, 2006 Paul Pogonyshev.                       *
  *                                                                 *
  * This program is free software; you can redistribute it and/or   *
  * modify it under the terms of the GNU General Public License as  *
@@ -120,6 +120,89 @@ sgf_undo_history_delete (SgfUndoHistory *history, SgfGameTree *tree)
 }
 
 
+void
+sgf_undo_history_hide_last_applied_entry (SgfUndoHistory *history)
+{
+  assert (history);
+  assert (history->last_applied_entry);
+
+  history->last_applied_entry->is_hidden = 1;
+}
+
+
+int
+sgf_undo_history_is_last_applied_entry_single (const SgfUndoHistory *history)
+{
+  assert (history);
+
+  if (history->last_applied_entry) {
+    return (history->last_applied_entry->is_last_in_action
+	    && (!history->last_applied_entry->previous
+		|| history->last_applied_entry->previous->is_last_in_action));
+  }
+  else
+    return 0;
+}
+
+
+int
+sgf_undo_history_check_last_applied_custom_entry_type
+  (const SgfUndoHistory *history,
+   const SgfCustomUndoHistoryEntryData *entry_data)
+{
+  const SgfCustomOperationEntry *custom_entry;
+
+  assert (history);
+  assert (entry_data);
+
+  custom_entry = (const SgfCustomOperationEntry *) history->last_applied_entry;
+
+  return (custom_entry
+	  && custom_entry->entry.operation_index == SGF_OPERATION_CUSTOM
+	  && custom_entry->entry_data == entry_data);
+}
+
+
+void *
+sgf_undo_history_get_last_applied_custom_entry_data
+  (const SgfUndoHistory *history)
+{
+  assert (history);
+  assert (history->last_applied_entry);
+  assert (history->last_applied_entry->operation_index
+	  == SGF_OPERATION_CUSTOM);
+
+  return (((const SgfCustomOperationEntry *) history->last_applied_entry)
+	  ->user_data);
+}
+
+
+void
+sgf_undo_history_delete_redo_entries (SgfUndoHistory *history,
+				      SgfGameTree *tree)
+{
+  SgfUndoHistoryEntry *this_entry;
+
+  assert (history);
+
+  if (history->last_applied_entry) {
+    this_entry			      = history->last_applied_entry->next;
+    history->last_applied_entry->next = NULL;
+  }
+  else {
+    this_entry		 = history->first_entry;
+    history->first_entry = NULL;
+  }
+
+  while (this_entry) {
+    SgfUndoHistoryEntry *next_entry = this_entry->next;
+
+    delete_undo_history_entry (this_entry, 0, tree);
+    this_entry = next_entry;
+  }
+}
+
+
 void
 sgf_undo_history_set_notification_callback
   (SgfUndoHistory *history,
@@ -176,6 +259,7 @@ void
 sgf_utils_undo (SgfGameTree *tree)
 {
   SgfUndoHistoryEntry *entry;
+  int is_hidden_entry = 0;
 
   assert (tree);
   assert (tree->board_state);
@@ -189,9 +273,12 @@ sgf_utils_undo (SgfGameTree *tree)
   entry = tree->undo_history->last_applied_entry;
 
   do {
+    if (entry->is_last_in_action)
+      is_hidden_entry = entry->is_hidden;
+
     sgf_undo_operations[entry->operation_index].undo (entry, tree);
     entry = entry->previous;
-  } while (entry && !entry->is_last_in_action);
+  } while (entry && (!entry->is_last_in_action || is_hidden_entry));
 
   tree->undo_history->last_applied_entry = entry;
   end_undoing_or_redoing (tree);
@@ -219,7 +306,8 @@ sgf_utils_redo (SgfGameTree *tree)
 
   while (1) {
     sgf_undo_operations[entry->operation_index].redo (entry, tree);
-    if (entry->is_last_in_action)
+    if (entry->is_last_in_action
+	&& (!entry->is_hidden || !entry->next))
       break;
 
     entry = entry->next;
@@ -285,15 +373,31 @@ sgf_delete_node_children_undo_history_entry_new (SgfNode *node)
 
 
 SgfUndoHistoryEntry *
-sgf_change_node_move_color_undo_history_entry_new (SgfNode *node,
-						   int new_move_color)
+sgf_swap_nodes_undo_history_entry_new (SgfNode *node1, SgfNode *node2)
 {
-  SgfChangeNodeMoveColorOperationEntry *operation_data
-    = utils_malloc (sizeof (SgfChangeNodeMoveColorOperationEntry));
+  SgfTwoNodesOperationEntry *operation_data
+    = utils_malloc (sizeof (SgfTwoNodesOperationEntry));
 
-  operation_data->entry.operation_index = SGF_OPERATION_CHANGE_NODE_MOVE_COLOR;
+  operation_data->entry.operation_index	   = SGF_OPERATION_SWAP_NODES;
+  operation_data->node1			   = node1;
+  operation_data->node2			   = node2;
+  operation_data->parent_current_variation = node1->parent->current_variation;
+
+  return (SgfUndoHistoryEntry *) operation_data;
+}
+
+
+SgfUndoHistoryEntry *
+sgf_change_node_inlined_color_undo_history_entry_new
+  (SgfNode *node, SgfUndoOperation operation, int new_color, int side_effect)
+{
+  SgfChangeNodeInlinedColorOperationEntry *operation_data
+    = utils_malloc (sizeof (SgfChangeNodeInlinedColorOperationEntry));
+
+  operation_data->entry.operation_index = operation;
   operation_data->node			= node;
-  operation_data->move_color		= new_move_color;
+  operation_data->color			= new_color;
+  operation_data->side_effect		= side_effect;
 
   return (SgfUndoHistoryEntry *) operation_data;
 }
@@ -301,7 +405,8 @@ sgf_change_node_move_color_undo_history_entry_new (SgfNode *node,
 
 SgfUndoHistoryEntry *
 sgf_new_property_undo_history_entry_new (SgfGameTree *tree, SgfNode *node,
-					 SgfProperty **link, SgfType type)
+					 SgfProperty **link, SgfType type,
+					 int side_effect)
 {
   SgfPropertyOperationEntry *operation_data
     = utils_malloc (sizeof (SgfPropertyOperationEntry));
@@ -309,6 +414,7 @@ sgf_new_property_undo_history_entry_new (SgfGameTree *tree, SgfNode *node,
   operation_data->entry.operation_index = SGF_OPERATION_NEW_PROPERTY;
   operation_data->node			= node;
   operation_data->property		= sgf_property_new (tree, type, *link);
+  operation_data->side_effect		= side_effect;
 
   return (SgfUndoHistoryEntry *) operation_data;
 }
@@ -316,7 +422,8 @@ sgf_new_property_undo_history_entry_new (SgfGameTree *tree, SgfNode *node,
 
 SgfUndoHistoryEntry *
 sgf_delete_property_undo_history_entry_new (SgfNode *node,
-					    SgfProperty *property)
+					    SgfProperty *property,
+					    int side_effect)
 {
   SgfPropertyOperationEntry *operation_data
     = utils_malloc (sizeof (SgfPropertyOperationEntry));
@@ -324,6 +431,7 @@ sgf_delete_property_undo_history_entry_new (SgfNode *node,
   operation_data->entry.operation_index = SGF_OPERATION_DELETE_PROPERTY;
   operation_data->node			= node;
   operation_data->property		= property;
+  operation_data->side_effect		= side_effect;
 
   return (SgfUndoHistoryEntry *) operation_data;
 }
@@ -331,7 +439,8 @@ sgf_delete_property_undo_history_entry_new (SgfNode *node,
 
 SgfUndoHistoryEntry *
 sgf_change_property_undo_history_entry_new (SgfNode *node,
-					    SgfProperty *property)
+					    SgfProperty *property,
+					    int side_effect)
 {
   SgfChangePropertyOperationEntry *operation_data
     = utils_malloc (sizeof (SgfChangePropertyOperationEntry));
@@ -339,6 +448,7 @@ sgf_change_property_undo_history_entry_new (SgfNode *node,
   operation_data->entry.operation_index = SGF_OPERATION_CHANGE_PROPERTY;
   operation_data->node			= node;
   operation_data->property		= property;
+  operation_data->side_effect		= side_effect;
 
   return (SgfUndoHistoryEntry *) operation_data;
 }
@@ -349,7 +459,8 @@ sgf_change_property_undo_history_entry_new (SgfNode *node,
 SgfUndoHistoryEntry *
 sgf_change_real_property_undo_history_entry_new (SgfNode *node,
 						 SgfProperty *property,
-						 double new_value)
+						 double new_value,
+						 int side_effect)
 {
   SgfChangeRealPropertyOperationEntry *operation_data
     = utils_malloc (sizeof (SgfChangeRealPropertyOperationEntry));
@@ -358,6 +469,7 @@ sgf_change_real_property_undo_history_entry_new (SgfNode *node,
   operation_data->node			= node;
   operation_data->property		= property;
   operation_data->value			= new_value;
+  operation_data->side_effect		= side_effect;
 
   return (SgfUndoHistoryEntry *) operation_data;
 }
@@ -365,6 +477,23 @@ sgf_change_real_property_undo_history_entry_new (SgfNode *node,
 #endif /* SGF_REAL_VALUES_ALLOCATED_SEPARATELY */
 
 
+SgfUndoHistoryEntry *
+sgf_custom_undo_history_entry_new
+  (const SgfCustomUndoHistoryEntryData *entry_data, void *user_data,
+   SgfNode *node_to_switch_to)
+{
+  SgfCustomOperationEntry *operation_data
+    = utils_malloc (sizeof (SgfCustomOperationEntry));
+
+  operation_data->entry.operation_index = SGF_OPERATION_CUSTOM;
+  operation_data->entry_data		= entry_data;
+  operation_data->user_data		= user_data;
+  operation_data->node_to_switch_to	= node_to_switch_to;
+
+  return (SgfUndoHistoryEntry *) operation_data;
+}
+
+
 inline static void
 delete_undo_history_entry (SgfUndoHistoryEntry *entry, int is_applied,
 			   SgfGameTree *tree)
@@ -378,6 +507,8 @@ delete_undo_history_entry (SgfUndoHistoryEntry *entry, int is_applied,
 }
 
 
+
+
 static void
 begin_undoing_or_redoing (SgfGameTree *tree)
 {
@@ -468,17 +599,7 @@ sgf_utils_apply_undo_history_entry (SgfGameTree *tree,
   SgfUndoHistory *history = tree->undo_history;
 
   if (history) {
-    SgfUndoHistoryEntry *this_entry;
-
-    for (this_entry = (history->last_applied_entry
-		       ? history->last_applied_entry->next
-		       : history->first_entry);
-	 this_entry;) {
-      SgfUndoHistoryEntry *next_entry = this_entry->next;
-
-      delete_undo_history_entry (this_entry, 0, tree);
-      this_entry = next_entry;
-    }
+    sgf_undo_history_delete_redo_entries (history, tree);
 
     if (history->last_applied_entry) {
       history->last_applied_entry->next = entry;
@@ -493,6 +614,7 @@ sgf_utils_apply_undo_history_entry (SgfGameTree *tree,
     history->last_applied_entry = entry;
     entry->next			= NULL;
     entry->is_last_in_action	= 0;
+    entry->is_hidden		= 0;
   }
 
   /* Perform the operation. */
@@ -506,6 +628,8 @@ sgf_utils_apply_undo_history_entry (SgfGameTree *tree,
 }
 
 
+
+
 void
 sgf_operation_add_node (SgfUndoHistoryEntry *entry, SgfGameTree *tree)
 {
@@ -614,6 +738,52 @@ sgf_operation_delete_node_children_free_data (SgfUndoHistoryEntry *entry,
 }
 
 
+void
+sgf_operation_swap_nodes_do_swap (SgfUndoHistoryEntry *entry, 
+				  SgfGameTree *tree)
+{
+  SgfNode *node1  = ((SgfTwoNodesOperationEntry *) entry)->node1;
+  SgfNode *node2  = ((SgfTwoNodesOperationEntry *) entry)->node2;
+  SgfNode *parent = node1->parent;
+  SgfNode *temp;
+  SgfNode **link;
+
+  assert (node1->parent == node2->parent);
+
+  set_is_modifying_map (tree);
+  set_is_modifying_tree (tree);
+
+  tree->node_to_switch_to = node1;
+
+  link = &parent->child;
+  while (*link != node1 && *link != node2) {
+    assert (*link);
+    link = & (*link)->next;
+  }
+
+  if (*link == node2) {
+    temp  = node1;
+    node1 = node2;
+    node2 = temp;
+  }
+
+  *link = node2;
+
+  temp	      = node1->next;
+  node1->next = node2->next;
+  node2->next = temp;
+
+  do {
+    assert (*link);
+    link = & (*link)->next;
+  } while (*link != node2);
+
+  *link = node1;
+
+  sgf_game_tree_invalidate_map (tree, parent);
+}
+
+
 /* Works as both undo and redo handler.  Since we just swap move
  * colors between the node and the undo history entry, it will always
  * do the right thing.
@@ -622,16 +792,40 @@ void
 sgf_operation_change_node_move_color_do_change (SgfUndoHistoryEntry *entry,
 						SgfGameTree *tree)
 {
-  SgfNode *node = ((SgfChangeNodeMoveColorOperationEntry *) entry)->node;
-  int *move_color
-    = & ((SgfChangeNodeMoveColorOperationEntry *) entry)->move_color;
-  int temp_move_color;
+  SgfChangeNodeInlinedColorOperationEntry *color_entry
+    = (SgfChangeNodeInlinedColorOperationEntry *) entry;
+  int temp_color;
 
-  UNUSED (tree);
+  if (!color_entry->side_effect)
+    tree->node_to_switch_to = color_entry->node;
 
-  temp_move_color  = node->move_color;
-  node->move_color = *move_color;
-  *move_color	   = temp_move_color;
+  temp_color			= color_entry->node->move_color;
+  color_entry->node->move_color = color_entry->color;
+  color_entry->color		= temp_color;
+}
+
+
+/* Works as both undo and redo handler.  Since we just swap move
+ * colors between the node and the undo history entry, it will always
+ * do the right thing.
+ */
+void
+sgf_operation_change_node_to_play_color_do_change (SgfUndoHistoryEntry *entry,
+						   SgfGameTree *tree)
+{
+  SgfChangeNodeInlinedColorOperationEntry *color_entry
+    = (SgfChangeNodeInlinedColorOperationEntry *) entry;
+  SgfNode *node = (SgfNode *) color_entry->node;
+  int temp_color;
+
+  tree->node_to_switch_to = node;
+
+  temp_color	      = node->to_play_color;
+  node->to_play_color = color_entry->color;
+  color_entry->color  = temp_color;
+
+  if (node == tree->current_node)
+    sgf_utils_find_board_state_data (tree, 1, 0);
 }
 
 
@@ -641,7 +835,8 @@ sgf_operation_add_property (SgfUndoHistoryEntry *entry, SgfGameTree *tree)
   SgfNode     *node	= ((SgfPropertyOperationEntry *) entry)->node;
   SgfProperty *property = ((SgfPropertyOperationEntry *) entry)->property;
 
-  tree->node_to_switch_to = node;
+  if (!((SgfPropertyOperationEntry *) entry)->side_effect)
+    tree->node_to_switch_to = node;
 
   * find_property_link (node, property->next) = property;
 }
@@ -653,7 +848,8 @@ sgf_operation_delete_property (SgfUndoHistoryEntry *entry, SgfGameTree *tree)
   SgfNode     *node	= ((SgfPropertyOperationEntry *) entry)->node;
   SgfProperty *property = ((SgfPropertyOperationEntry *) entry)->property;
 
-  tree->node_to_switch_to = node;
+  if (!((SgfPropertyOperationEntry *) entry)->side_effect)
+    tree->node_to_switch_to = node;
 
   * find_property_link (node, property) = property->next;
 }
@@ -695,7 +891,8 @@ sgf_operation_change_property_do_change (SgfUndoHistoryEntry *entry,
   SgfProperty *property = change_entry->property;
   SgfValue    *value	= &change_entry->value;
 
-  tree->node_to_switch_to = node;
+  if (!change_entry->side_effect)
+    tree->node_to_switch_to = node;
 
   switch (property_info[property->type].value_type) {
   case SGF_NUMBER:
@@ -815,7 +1012,8 @@ sgf_operation_change_real_property_do_change (SgfUndoHistoryEntry *entry,
   SgfProperty *property = change_entry->property;
   double temp_value;
 
-  tree->node_to_switch_to = node;
+  if (!change_entry->side_effect)
+    tree->node_to_switch_to = node;
 
   temp_value		= *property->value.real;
   *property->value.real = change_entry->value;
@@ -825,6 +1023,45 @@ sgf_operation_change_real_property_do_change (SgfUndoHistoryEntry *entry,
 #endif
 
 
+void
+sgf_operation_custom_undo (SgfUndoHistoryEntry *entry, SgfGameTree *tree)
+{
+  SgfCustomOperationEntry *custom_entry = (SgfCustomOperationEntry *) entry;
+
+  if (custom_entry->entry_data->undo)
+    custom_entry->entry_data->undo (custom_entry->user_data, tree);
+
+  if (custom_entry->node_to_switch_to)
+    tree->node_to_switch_to = custom_entry->node_to_switch_to;
+}
+
+
+void
+sgf_operation_custom_redo (SgfUndoHistoryEntry *entry, SgfGameTree *tree)
+{
+  SgfCustomOperationEntry *custom_entry = (SgfCustomOperationEntry *) entry;
+
+  if (custom_entry->entry_data->redo)
+    custom_entry->entry_data->redo (custom_entry->user_data, tree);
+
+  if (custom_entry->node_to_switch_to)
+    tree->node_to_switch_to = custom_entry->node_to_switch_to;
+}
+
+
+void
+sgf_operation_custom_free_data  (SgfUndoHistoryEntry *entry, int is_applied,
+				 SgfGameTree *tree)
+{
+  SgfCustomOperationEntry *custom_entry = (SgfCustomOperationEntry *) entry;
+
+  UNUSED (is_applied);
+
+  if (custom_entry->entry_data->free_data)
+    custom_entry->entry_data->free_data (custom_entry->user_data, tree);
+}
+
+
 inline static SgfNode **
 find_node_link (SgfNode *parent, const SgfNode *next)
 {
